#ifdef HAVE_CONFIG_H
#include <config.h>

/* If autoconfed, only include malloc.h when it's presen */
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#else /* undef HAVE_CONFIG_H */

#include <malloc.h>

#endif /* HAVE_CONFIG_H */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include "grads.h"
#include <gadods.h>

/*  Open a DODS station data set and fill in the gafile
    info from metadata from the server */

int dodpfi (char *url, struct gafile *pfi) {
struct gavar *pvar;
struct dt tdef,dt1,dt2;
int i,j,k,nvars,nivars,size,lcnt,len;
const char *name;
GADODS_DATASET handle;
GADODS_STATUS rc;
float v1,v2,*vals;
double time,fill;
char *pos;


  rc = gadods_open(url, &handle);

  if (rc!=0) {
    gaprnt (0,"Open Error on DODS URL\n");
    return (99);
  }

  nvars = gadods_d_numvars(handle);
  if (nvars<1) {
    gaprnt (0,"Open error:  DODS URL is not a station dataset\n");
    gadods_close(handle,1);
    return(99);
  }

  pfi->type = 2;
  pfi->dhandle = handle;    /* qqq this could be a problem */
  pfi->dlev = -999;
  len = 0;
  while (*(url+len) && len<4095) {
    pfi->name[len] = *(url+len);
    pfi->dnam[len] = *(url+len);
    len++;
  }
  pfi->name[len] = '\0';  pfi->dnam[len] = '\0';

  name = gadods_d_title(handle);
  if (name==NULL) {
    name = gadods_d_attr(handle,0);
  }
  len = 0;
  while (*(name+len) && len<510) {
    pfi->title[len] = *(name+len);
    len++;
  }
  pfi->title[len] = '\0';

  nivars = gadods_d_numlivars(handle);

  size = nvars * (sizeof(struct gavar) + 7 );
  pvar = (struct gavar *)malloc(size);
  if (pvar==NULL) {
    gaprnt (0,"Memory allocation error in dodpfi\n");
    gadods_close(handle,1);
    return(99);
  }
  pfi->pvar1 = pvar;
  lcnt = 0;
  for (i=4; i<nvars; i++) {
    j = 0;
    if (i>=nivars) j = 1;
    name = gadods_d_varname(handle,i);
    k = 1;
    if (*name=='l' && *(name+1)=='e' && *(name+2)=='v' &&
        *(name+3)=='\0') {
       k = 0;
       pfi->dlev = i;
    }
    fill = -9.99e33;
    gadods_d_fill(handle,i,&fill);
    if (i==4) {
      pfi->undef = fill;
      if (SETMISS) {
        pfi->ulow = fabs(pfi->undef/EPSILON);
        pfi->uhi = pfi->undef + pfi->ulow;
        pfi->ulow = pfi->undef - pfi->ulow;
      }
    }
    if (k) {
      pvar->offset = i;
      pvar->units[0] = 99;
      pvar->levels = j;
      len = 0;
      while (*(name+len)) {
        pvar->varnm[len] = *(name+len);
        pvar->abbrv[len] = *(name+len);
        len++;
      }
      pvar->varnm[len] = '\0'; pvar->abbrv[len] = '\0';
      pvar++;
      lcnt++;
    }
  }
  pfi->vnum = lcnt;
  pfi->ivnum = nivars-4; 
  pfi->lvnum = lcnt - pfi->ivnum;
  if (pfi->lvnum>0 && pfi->dlev<0) goto leverr;

  /* Parse tdef info provided by server */

  i = gadods_d_attrindex(handle,"time.grads_size");
  if (i<0) goto tdeferr;
  name = gadods_d_attr(handle,i);
  if ( (pos = intprs((char *)name,&(pfi->dnum[3])))==NULL) goto tdeferr;

  i = gadods_d_attrindex(handle,"time.grads_min");
  if (i<0) goto tdeferr;
  name = gadods_d_attr(handle,i);
  tdef.yr = -1000;
  tdef.mo = -1000;
  tdef.dy = -1000;
  if ( (pos = adtprs((char *)name,&tdef,&dt1))==NULL) goto tdeferr;
  if (dt1.yr == -1000 || dt1.mo == -1000.0 || dt1.dy == -1000) goto tdeferr;

  i = gadods_d_attrindex(handle,"time.grads_step");
  if (i<0) goto tdeferr;
  name = gadods_d_attr(handle,i);
  if ( (pos = rdtprs((char *)name,&dt2))==NULL) goto tdeferr;
  v1 = (dt2.yr * 12) + dt2.mo;
  v2 = (dt2.dy * 1440) + (dt2.hr * 60) + dt2.mn;
  if( (v1 == 0) && (v2 == 0) ) goto tdeferr;

  /* The info we just collected gets hung off the pfi block
     as the time conversion constants */

  vals = (float *)malloc(sizeof(float)*8);
  if (vals==NULL) goto tdeferr;
  *(vals) = dt1.yr;
  *(vals+1) = dt1.mo;
  *(vals+2) = dt1.dy;
  *(vals+3) = dt1.hr;
  *(vals+4) = dt1.mn;
  *(vals+5) = v1;
  *(vals+6) = v2;
  *(vals+7) = -999.9;
  pfi->grvals[3] = vals;
  pfi->abvals[3] = vals;
  pfi->linear[3] = 1;

  return (0);

tdeferr:
  gaprnt (0,"Invalid tdef info from server; error in dodpfi\n");
  gadods_close(handle,1);
  return(99);
leverr:
  gaprnt (0,"Invalid lev info from server; error in dodpfi\n");
  gadods_close(handle,1);
  return(99);
}

/* Obtain data to satisfy the request described in the 
   gastn block */

int dodget(struct gastn *stn) {
struct gavar *pvar;
struct garpt *rpt;
struct dt dt;
GADODS_DATASET handle;
GADODS_STATUS rc;
GADODS_STN_QUERY *query;
GADODS_RPTCOL r_handle;
float lon,lat,lev,val;
double time,fill;
int nreps,nlevs,i,j,k,num;
char tchmn[20],tchmx[20],stid[10];
const char *stid2;

  handle = stn->pfi->dhandle;
  query = gadods_sq_new(handle);
  if (query==NULL) { 
    gaprnt (0,"Memory allocation error in dodget\n");
    return (99);
  }

  pvar = stn->pvar;

  query->varflags[0] = 1;
  query->varflags[1] = 1;
  query->varflags[2] = 1;
  query->varflags[3] = 1;
  query->varflags[pvar->offset] = 1;
  if (pvar->levels>0) {
    query->varflags[stn->pfi->dlev] = 1;
  }
  query->minlon = stn->dmin[0];
  query->maxlon = stn->dmax[0];
  query->minlat = stn->dmin[1];
  query->maxlat = stn->dmax[1];
  query->minlev = stn->dmin[2];
  query->maxlev = stn->dmax[2];
  gr2t (stn->tvals, stn->tmin, &dt);
  gat2ch (&dt, 4, tchmn);   /* should be 5 */
  query->mintime = tchmn;
  gr2t (stn->tvals, stn->tmax, &dt);
  gat2ch (&dt, 4, tchmx);
  query->maxtime = tchmx;
  if (stn->sflag) {
    for (i=0; i<8; i++) stid[i] = stn->stid[i];
    i = 0;
    while (stid[i]!=' ' && i<8) i++;
    stid[i] = '\0';
    query->stid = stid;
  } else {
    query->stid = NULL;
  }
  query->usebounds = 1;

  rc = gadods_sq_send(query, &r_handle);
  if (rc) {
    gaprnt (0,"DODS data retreival error\n");
    gadods_sq_free(query);
    return(99);
  }

  nreps = gadods_r_numrpts(r_handle);

  num = 0;
  for (i = 0; i < nreps; i++) {
    stid2 = gadods_r_stid(r_handle, i);
    gadods_r_time(r_handle, i, &time);
    gadods_r_lon(r_handle, i, &lon);
    gadods_r_lat(r_handle, i, &lat);
    if (pvar->levels==0) {
      gadods_r_var(r_handle, i, 0, 4, &val);
      rpt = gaarpt(stn);
      for (k=0; k<8; k++) rpt->stid[k] = *(stid2+k);  /* assume blanks? */
      rpt->lon = lon;
      rpt->lat = lat;
      rpt->tim = time;
      rpt->lev = -9.99e33;
      rpt->val = val;
      num++;
    } else {
      nlevs = gadods_r_numlev(r_handle, i);
      for (j=0; j<nlevs; j++) {
        gadods_r_var(r_handle, i, j, 5, &val);
        gadods_r_lev(r_handle, i, j, &lev);
        rpt = gaarpt(stn);
        for (k=0; k<8; k++) rpt->stid[k] = *(stid2+k);  /* assume blanks? */
        rpt->lon = lon;
        rpt->lat = lat;
        rpt->tim = time;
        rpt->lev = lev;
        rpt->val = val;
        num++;
      }
    }
  }
  stn->rnum = num;
  gadods_r_free(r_handle);
  gadods_sq_free(query);
  return (0);
}

/*  Close gadods data set */

void dodclo (struct gafile *pfi) {
GADODS_DATASET handle;

  if (pfi->dhandle == -999) return;
  handle = pfi->dhandle;
  printf ("qqq closing dods dataset\n");
  gadods_close(handle,1);
  pfi->dhandle = -999;
  printf ("qqq dods dataset has been closed\n");
}
